@@ -9,7 +9,7 @@
 """Extractors for https://www.tumblr.com/"""
 
 from .common import Extractor, Message
-from .. import text, oauth, exception
+from .. import text, util, oauth, exception
 from datetime import datetime, date, timedelta
 import re
 
@@ -262,7 +262,7 @@ class TumblrExtractor(Extractor):
             return updated, (resized == updated)
 
     def _original_image_fallback(self, url, post_id):
-        for _ in range(self.fallback_retries):
+        for _ in util.repeat(self.fallback_retries):
             self.sleep(self.fallback_delay, "image token")
             yield self._update_image_token(url)[0]
         self.log.warning("Unable to fetch higher-resolution "
@@ -386,7 +386,7 @@ class TumblrAPI(oauth.OAuth1API):
     def posts(self, blog, params):
         """Retrieve published posts"""
         params["offset"] = self.extractor.config("offset")
-        params["limit"] = "50"
+        params["limit"] = 50
         params["reblog_info"] = "true"
         params["type"] = self.posts_type
         params["before"] = self.before
@@ -398,78 +398,92 @@ class TumblrAPI(oauth.OAuth1API):
 
     def likes(self, blog):
         """Retrieve liked posts"""
+        endpoint = "/v2/blog/{}/likes".format(blog)
         params = {"limit": "50", "before": self.before}
-        return self._pagination(blog, "/likes", params, key="liked_posts")
+        if self.api_key:
+            params["api_key"] = self.api_key
+
+        while True:
+            posts = self._call(endpoint, params)["liked_posts"]
+            if not posts:
+                return
+            yield from posts
+            params["before"] = posts[-1]["liked_timestamp"]
 
     def _call(self, endpoint, params, **kwargs):
         url = self.ROOT + endpoint
         kwargs["params"] = params
-        response = self.request(url, **kwargs)
 
-        try:
-            data = response.json()
-        except ValueError:
-            data = response.text
-            status = response.status_code
-        else:
-            status = data["meta"]["status"]
-            if 200 <= status < 400:
-                return data["response"]
+        while True:
+            response = self.request(url, **kwargs)
 
-        self.log.debug(data)
-        if status == 403:
-            raise exception.AuthorizationError()
-
-        elif status == 404:
             try:
-                error = data["errors"][0]["detail"]
-                board = ("only viewable within the Tumblr dashboard" in error)
-            except Exception:
-                board = False
+                data = response.json()
+            except ValueError:
+                data = response.text
+                status = response.status_code
+            else:
+                status = data["meta"]["status"]
+                if 200 <= status < 400:
+                    return data["response"]
 
-            if board:
-                self.log.info("Run 'gallery-dl oauth:tumblr' "
-                              "to access dashboard-only blogs")
-                raise exception.AuthorizationError(error)
-            raise exception.NotFoundError("user or post")
+            self.log.debug(data)
 
-        elif status == 429:
-            # daily rate limit
-            if response.headers.get("x-ratelimit-perday-remaining") == "0":
-                self.log.info("Daily API rate limit exceeded")
-                reset = response.headers.get("x-ratelimit-perday-reset")
+            if status == 403:
+                raise exception.AuthorizationError()
 
-                api_key = self.api_key or self.session.auth.consumer_key
-                if api_key == self.API_KEY:
-                    self.log.info("Register your own OAuth application and "
-                                  "use its credentials to prevent this error: "
-                                  "https://github.com/mikf/gallery-dl/blob/mas"
-                                  "ter/docs/configuration.rst#extractortumblra"
-                                  "pi-key--api-secret")
+            elif status == 404:
+                try:
+                    error = data["errors"][0]["detail"]
+                    board = ("only viewable within the Tumblr dashboard"
+                             in error)
+                except Exception:
+                    board = False
 
-                if self.extractor.config("ratelimit") == "wait":
+                if board:
+                    self.log.info("Run 'gallery-dl oauth:tumblr' "
+                                  "to access dashboard-only blogs")
+                    raise exception.AuthorizationError(error)
+                raise exception.NotFoundError("user or post")
+
+            elif status == 429:
+                # daily rate limit
+                if response.headers.get("x-ratelimit-perday-remaining") == "0":
+                    self.log.info("Daily API rate limit exceeded")
+                    reset = response.headers.get("x-ratelimit-perday-reset")
+
+                    api_key = self.api_key or self.session.auth.consumer_key
+                    if api_key == self.API_KEY:
+                        self.log.info(
+                            "Register your own OAuth application and use its "
+                            "credentials to prevent this error: "
+                            "https://gdl-org.github.io/docs/configuration.html"
+                            "#extractor-tumblr-api-key-api-secret")
+
+                    if self.extractor.config("ratelimit") == "wait":
+                        self.extractor.wait(seconds=reset)
+                        continue
+
+                    t = (datetime.now() + timedelta(0, float(reset))).time()
+                    raise exception.StopExtraction(
+                        "Aborting - Rate limit will reset at %s",
+                        "{:02}:{:02}:{:02}".format(t.hour, t.minute, t.second))
+
+                # hourly rate limit
+                reset = response.headers.get("x-ratelimit-perhour-reset")
+                if reset:
+                    self.log.info("Hourly API rate limit exceeded")
                     self.extractor.wait(seconds=reset)
-                    return self._call(endpoint, params, **kwargs)
+                    continue
 
-                t = (datetime.now() + timedelta(seconds=float(reset))).time()
-                raise exception.StopExtraction(
-                    "Aborting - Rate limit will reset at %s",
-                    "{:02}:{:02}:{:02}".format(t.hour, t.minute, t.second))
-
-            # hourly rate limit
-            reset = response.headers.get("x-ratelimit-perhour-reset")
-            if reset:
-                self.log.info("Hourly API rate limit exceeded")
-                self.extractor.wait(seconds=reset)
-                return self._call(endpoint, params, **kwargs)
-
-        raise exception.StopExtraction(data)
+            raise exception.StopExtraction(data)
 
     def _pagination(self, blog, endpoint, params, key="posts", cache=False):
         endpoint = "/v2/blog/{}{}".format(blog, endpoint)
         if self.api_key:
             params["api_key"] = self.api_key
 
+        strategy = self.extractor.config("pagination")
         while True:
             data = self._call(endpoint, params)
 
@@ -477,13 +491,31 @@ class TumblrAPI(oauth.OAuth1API):
                 self.BLOG_CACHE[blog] = data["blog"]
                 cache = False
 
-            yield from data[key]
+            posts = data[key]
+            yield from posts
 
-            try:
-                endpoint = data["_links"]["next"]["href"]
-            except KeyError:
-                return
+            if strategy == "api":
+                try:
+                    endpoint = data["_links"]["next"]["href"]
+                except KeyError:
+                    return
 
-            params = None
-            if self.api_key:
-                endpoint += "&api_key=" + self.api_key
+                params = None
+                if self.api_key:
+                    endpoint += "&api_key=" + self.api_key
+
+            elif strategy == "before":
+                if not posts:
+                    return
+                timestamp = posts[-1]["timestamp"] + 1
+                if params["before"] and timestamp >= params["before"]:
+                    return
+                params["before"] = timestamp
+                params["offset"] = None
+
+            else:  # offset
+                params["offset"] = \
+                    text.parse_int(params["offset"]) + params["limit"]
+                params["before"] = None
+                if params["offset"] >= data["total_posts"]:
+                    return
